/*
 *
 * Copyright (c) 2019 Texas Instruments Incorporated
 *
 * All rights reserved not granted herein.
 *
 * Limited License.
 *
 * Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive
 * license under copyrights and patents it now or hereafter owns or controls to make,
 * have made, use, import, offer to sell and sell ("Utilize") this software subject to the
 * terms herein.  With respect to the foregoing patent license, such license is granted
 * solely to the extent that any such patent is necessary to Utilize the software alone.
 * The patent license shall not apply to any combinations which include this software,
 * other than combinations with devices manufactured by or for TI ("TI Devices").
 * No hardware patent is licensed hereunder.
 *
 * Redistributions must preserve existing copyright notices and reproduce this license
 * (including the above copyright notice and the disclaimer and (if applicable) source
 * code license limitations below) in the documentation and/or other materials provided
 * with the distribution
 *
 * Redistribution and use in binary form, without modification, are permitted provided
 * that the following conditions are met:
 *
 * *       No reverse engineering, decompilation, or disassembly of this software is
 * permitted with respect to any software provided in binary form.
 *
 * *       any redistribution and use are licensed by TI for use only with TI Devices.
 *
 * *       Nothing shall obligate TI to provide you with source code for the software
 * licensed and provided to you in object code.
 *
 * If software source code is provided to you, modification and redistribution of the
 * source code are permitted provided that the following conditions are met:
 *
 * *       any redistribution and use of the source code, including any resulting derivative
 * works, are licensed by TI for use only with TI Devices.
 *
 * *       any redistribution and use of any object code compiled from the source code
 * and any resulting derivative works, are licensed by TI for use only with TI Devices.
 *
 * Neither the name of Texas Instruments Incorporated nor the names of its suppliers
 *
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * DISCLAIMER.
 *
 * THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <string.h>
#include <utils/console_io/include/app_log.h>
#include <utils/remote_service/include/app_remote_service.h>
#include <utils/mem/include/app_mem.h>
#include <utils/ipc/include/app_ipc.h>

/* #define APP_REMOTE_SERVICE_TEST_DEBUG */
#define MY_REMOTE_SERVICE_TEST_NAME  "com.my.remote_service_test"

#define APP_REMOTE_SERVICE_TEST_CMD_0   (0x1234)
#define APP_REMOTE_SERVICE_TEST_CMD_1   (0x5678)

static int32_t myRemoteServiceTestHandler(char *service_name, uint32_t cmd, void *prm, uint32_t prm_size, uint32_t flags)
{
    uint32_t *value;

    #ifdef APP_REMOTE_SERVICE_TEST_DEBUG
    appLogPrintf("REMOTE_SERVICE_TEST: %s service received 0x%08x command and parameters of size %d bytes with %08x flags\n",
        service_name,
        cmd,
        prm_size,
        flags
        );
    #endif
    if(cmd==APP_REMOTE_SERVICE_TEST_CMD_0)
    {
        if(prm!=NULL)
        {
            value = (uint32_t*)prm;

            *value = *value + 1;
        }
    }
    if(cmd==APP_REMOTE_SERVICE_TEST_CMD_1)
    {
        /* prm is a physical pointer, it will be written with a value by the sender
         * we just print the value here and also echo back the same + 1 */
        if(prm!=NULL && prm_size == sizeof(uint32_t))
        {
            uintptr_t addr = (uintptr_t)(*(volatile uint32_t*)prm);
            uint32_t value = *(volatile uint32_t*)addr;

            appMemCacheInv((void*)addr, 256);

            #ifdef APP_REMOTE_SERVICE_TEST_DEBUG
            appLogPrintf("REMOTE_SERVICE_TEST: 0x%08x = %d\n",
                (uint32_t)addr,
                (uint32_t)value
                );
            #endif

            *(volatile uint32_t*)addr = value+1;

            appMemCacheWbInv((void*)addr, 256);
        }
    }
    return 0;
}

int32_t myRemoteServiceTestInit()
{
    int32_t status = 0;

    status = appRemoteServiceRegister(MY_REMOTE_SERVICE_TEST_NAME, myRemoteServiceTestHandler);
    if(status!=0)
    {
        appLogPrintf("REMOTE_SERVICE_TEST: ERROR: Unable to register remote service test handler\n");
    }
    return status;
}

int32_t myRemoteServiceTestDeInit()
{
    int32_t status = 0;

    status = appRemoteServiceUnRegister(MY_REMOTE_SERVICE_TEST_NAME);
    if(status!=0)
    {
        appLogPrintf("REMOTE_SERVICE_TEST: ERROR: Unable to register remote service test handler\n");
    }
    return status;
}

