#
# Utility makefile to build TIOVX, PTK and imaging libaries and related components
#
# Edit this file to suit your specific build needs
#

ifeq ($(PROFILE), $(filter $(PROFILE),debug all))
LINUX_APP_PROFILE=debug
endif
ifeq ($(PROFILE), $(filter $(PROFILE),release all))
LINUX_APP_PROFILE=release
endif

linux_fs_install:
ifeq ($(BUILD_CPU_MPU1),yes)
	# copy application binaries and scripts
	mkdir -p $(LINUX_FS_PATH)/opt/vision_apps
	cp $(MY_APPS_PATH)/out/J7/A72/LINUX/$(LINUX_APP_PROFILE)/*.out $(LINUX_FS_PATH)/opt/vision_apps
endif

	# remove old remote firmware files from filesystem
	mkdir -p $(LINUX_FS_PATH)/lib/firmware
	-rm $(LINUX_FS_PATH)/lib/firmware/j7-*-fw -f

ifeq ($(BUILD_CPU_MCU1_0),yes)
	# copy remote firmware files for mcu1_0
	cp $(MY_APPS_PATH)/out/J7/R5F/SYSBIOS/$(LINUX_APP_PROFILE)/my_app_tirtos_linux_mcu1_0.out $(LINUX_FS_PATH)/lib/firmware/j7-mcu-r5f0_0-fw
	$(TIARMCGT_ROOT)/bin/armstrip -p $(LINUX_FS_PATH)/lib/firmware/j7-mcu-r5f0_0-fw
endif
ifeq ($(BUILD_CPU_MCU1_1),yes)
	# copy remote firmware files for mcu1_1
	cp $(MY_APPS_PATH)/out/J7/R5F/SYSBIOS/$(LINUX_APP_PROFILE)/my_app_tirtos_linux_mcu1_1.out $(LINUX_FS_PATH)/lib/firmware/j7-mcu-r5f0_1-fw
	$(TIARMCGT_ROOT)/bin/armstrip -p $(LINUX_FS_PATH)/lib/firmware/j7-mcu-r5f0_1-fw
endif
ifeq ($(BUILD_CPU_MCU2_0),yes)
	# copy remote firmware files for mcu2_0
	cp $(MY_APPS_PATH)/out/J7/R5F/SYSBIOS/$(LINUX_APP_PROFILE)/my_app_tirtos_linux_mcu2_0.out $(LINUX_FS_PATH)/lib/firmware/j7-main-r5f0_0-fw
	$(TIARMCGT_ROOT)/bin/armstrip -p $(LINUX_FS_PATH)/lib/firmware/j7-main-r5f0_0-fw
endif
ifeq ($(BUILD_CPU_MCU2_1),yes)
	# copy remote firmware files for mcu2_1
	cp $(MY_APPS_PATH)/out/J7/R5F/SYSBIOS/$(LINUX_APP_PROFILE)/my_app_tirtos_linux_mcu2_1.out $(LINUX_FS_PATH)/lib/firmware/j7-main-r5f0_1-fw
	$(TIARMCGT_ROOT)/bin/armstrip -p $(LINUX_FS_PATH)/lib/firmware/j7-main-r5f0_1-fw
endif
ifeq ($(BUILD_CPU_MCU3_0),yes)
	# copy remote firmware files for mcu3_0
	cp $(MY_APPS_PATH)/out/J7/R5F/SYSBIOS/$(LINUX_APP_PROFILE)/my_app_tirtos_linux_mcu3_0.out $(LINUX_FS_PATH)/lib/firmware/j7-main-r5f1_0-fw
	$(TIARMCGT_ROOT)/bin/armstrip -p $(LINUX_FS_PATH)/lib/firmware/j7-main-r5f1_0-fw
endif
ifeq ($(BUILD_CPU_MCU3_1),yes)
	# copy remote firmware files for mcu3_1
	cp $(MY_APPS_PATH)/out/J7/R5F/SYSBIOS/$(LINUX_APP_PROFILE)/my_app_tirtos_linux_mcu3_1.out $(LINUX_FS_PATH)/lib/firmware/j7-main-r5f1_1-fw
	$(TIARMCGT_ROOT)/bin/armstrip -p $(LINUX_FS_PATH)/lib/firmware/j7-main-r5f1_1-fw
endif
ifeq ($(BUILD_CPU_C6x_1),yes)
	# copy remote firmware files for c6x_1
	cp $(MY_APPS_PATH)/out/J7/C66/SYSBIOS/$(LINUX_APP_PROFILE)/my_app_tirtos_linux_c6x_1.out $(LINUX_FS_PATH)/lib/firmware/j7-c66_0-fw
	$(CGT6X_ROOT)/bin/strip6x -p $(LINUX_FS_PATH)/lib/firmware/j7-c66_0-fw
endif
ifeq ($(BUILD_CPU_C6x_2),yes)
	# copy remote firmware files for c6x_2
	cp $(MY_APPS_PATH)/out/J7/C66/SYSBIOS/$(LINUX_APP_PROFILE)/my_app_tirtos_linux_c6x_2.out $(LINUX_FS_PATH)/lib/firmware/j7-c66_1-fw
	$(CGT6X_ROOT)/bin/strip6x -p $(LINUX_FS_PATH)/lib/firmware/j7-c66_1-fw
endif
ifeq ($(BUILD_CPU_C7x_1),yes)
	# copy remote firmware files for c7x_1
	cp $(MY_APPS_PATH)/out/J7/C71/SYSBIOS/$(LINUX_APP_PROFILE)/my_app_tirtos_linux_c7x_1.out $(LINUX_FS_PATH)/lib/firmware/j7-c71_0-fw
	$(CGT7X_ROOT)/bin/strip7x -p $(LINUX_FS_PATH)/lib/firmware/j7-c71_0-fw
endif

	sync


linux_fs_install_sd: linux_fs_install
	# copy vision apps binaries
	mkdir -p $(LINUX_SD_FS_ROOT_PATH)/opt/vision_apps
	cp -r $(LINUX_FS_PATH)/opt/vision_apps $(LINUX_SD_FS_ROOT_PATH)/opt/

	# copy remote core firmware's
	mkdir -p $(LINUX_SD_FS_ROOT_PATH)/lib/firmware
	-rm $(LINUX_SD_FS_ROOT_PATH)/lib/firmware/j7-*-fw -f
	cp $(LINUX_FS_PATH)/lib/firmware/j7-*-fw $(LINUX_SD_FS_ROOT_PATH)/lib/firmware/

	sync

